(function(){
	let appTpl = `<div class="feynotes-wrapper" id="feynotes-wrapper" style="display:none">
		<div class="feynote-panel-fold" v-if="feynType == 'close'">
			<span class="feynote-count" v-if="notes.length > 0">{{ notes.length }}</span>
			<span class="feynote-button" @click="transPanel()">Feynman 笔记</span>
		</div>
		<div class="feynotes" id="feynote-canvas" v-else>
			<template v-if="feynType == 'feynote'">
				<div class="feynote-banner" @click="setBanner(true)">
					<img :src="article.banner" alt="" v-if="article.banner">
				</div>
				<div class="feynote-header">
					<h2>{{ article.title || 'Feynman 笔记'}}</h2>
				</div>
				<div class="feynote-main">
					<div class="feynote-item" v-for="(note, index) in notes" :key="index" _images="note.images">
						<div class="feynote-item-content" @click="scrollToHL(note)">
							{{ note.text }}
						</div>
						<div class="feynote-item-remark" v-if="note.tip" @click="scrollToHL(note)">
							{{ note.tip }}
						</div>
					</div>
				</div>
				
				<div class="feynote-tags">
					文章标签 <input type="text" v-model="article.tag" placeholder="逗号分隔">
				</div>
				<div class="feynote-foot">
					<span @click="downloadNotes">导出笔记</span>
					<span @click="downloadArticle">下载原文</span>
					<span @click="createPublicLink">分享</span>
					<a href="http://notes.bluetech.top/public/index.html" target="_blank" v-if="token != ''">管理</a>
					<span @click="logout" v-if="token != ''">退出</span>
					<span @click="transPanel('close')">关闭</span>
				</div>
			</template>
			<template v-if="feynType == 'weread'">
				<div class="feynote-header">
					<h2>完读书籍</h2>
				</div>
				<div class="feynote-main feynote-books">
					<div v-for="(item, index) in books" :key="index" class="feynote-book-item">
						<h3> {{ item.book.title }} </h3>
						<p>作者：【{{ item.book.author }}】</p>
						<p>
							书摘数量：{{ item.noteCount }} 
							<span @click="downloadWeReadBook(item)">导出笔记</span>
							<span @click="shareWeReadBook(item)">分享</span>
						</p>
					</div>
				</div>
				<div class="feynote-foot">
					<a href="http://notes.bluetech.top/public/index.html" target="_blank" v-if="token != ''">管理</a>
					<span @click="logout" v-if="token != ''">退出</span>
					<span @click="transPanel('close')">关闭</span>
				</div>
			</template>
			<div class="feynote-login" v-if="isLogin">
				<div class="feynote-tags"><input type="text" v-model="userInfo.account" placeholder="账号"></div>
				<div class="feynote-tags"><input type="password" v-model="userInfo.password" placeholder="密码" @change="userLoginSubmit"></div>
				<div class="feynote-tags">
					<span class="feynote-button" @click="userLoginSubmit">登录</span>
					<span @click="isLogin = false">取消</span>
				</div>
			</div>
		</div>
	</div>`;

	setTimeout(function(){
		runFeynmanNote();
	},100);

	function runFeynmanNote(){
		//todo bugfix
		if(!window.Vue){
			return;
		}

		let noAllowDomains = [
			'pages.bluetech.top',
			'notes.bluetech.top'
		];

		//禁止注入，后期服务器端配置
		let _host = location.host;
		if(noAllowDomains.indexOf(_host) > -1){
			return;
		}

		//防止重复注入
		if(document.querySelector('#feynotes-wrapper')){
			return;	
		}
		let app = document.createElement('div');
			app.innerHTML = appTpl;
		document.body.appendChild(app);
		setTimeout(function(){
			document.querySelector('#feynotes-wrapper').style.display = 'block';
		}, 100);

		//获取用户登录信息
		feynmanRequest({
			type: 'getCookie',
			name: 'userInfo'
		}, function(cookie){
			cookie = cookie || {};
			let userInfo = JSON.parse(cookie.value || '{}');
			console.log('init form cache', userInfo);
			start(userInfo);
		});
	}

	function feynmanRequest(data, callback){
	    //发送创建请求
	    chrome.extension.sendRequest(data, function(response) {
	        callback(response);
	    });
	}

	function start(userInfo){
		new Vue({
			el: '#feynotes-wrapper',
			data: function(){
				return {
					feynType: 'close',
					isDraging: false,
					isLogin: false,
					userInfo: {
						account: userInfo.account || '',
						password: ''
					},
					token: userInfo.token || '',
					tags: [],
					pageId: '',
					banners: {
						index: -1,
						data: []
					},
					article: {
						title: '',
						tag: '',
						banner: '',
						originUrl: ''
					},
					notes: [],
					books: []
				}
			},
			created: function(){
				let url = location.href;
				this.pageId = md5(url);
				this.loadTags(userInfo);
			},
			mounted: function(){
				let _this = this;
				this.initPagenote();
				this.setArticleInfo();
				let t = setInterval(function(){
					if(_this.article.banner.indexOf('http') == 0){
						clearInterval(t);
					}
					_this.setArticleInfo();
				},5);
			},
			methods: {
				transPanel: function(type){
					let pageName = 'feynote';
					if(location.host == 'weread.qq.com'){
						pageName = 'weread';
						this.loadBooks();
					}
					
					this.feynType = type || pageName;
				},
				loadBooks: function(callback){
					let _this = this;
					feynmanRequest({
						api: 'weRead.books',
						type: 'request'
					}, function(res){
						console.log('weRead.books', res.books);
						_this.books = res.books;
					});
				},
				loadBookNotes: function(book, callback){
					let _this = this;
					let count = 0;
					let notes = null, summarys = null;
					feynmanRequest({
						api: 'weRead.notes',
						type: 'request',
						data: book
					}, function(res){
						console.log('weRead.notes', res);
						count += 1;
						notes = res;
						callbacks();
						// callback(res.books);
					});

					feynmanRequest({
						api: 'weRead.summarys',
						type: 'request',
						data: book
					}, function(res){
						console.log('weRead.summarys', res);
						count += 1;
						summarys = res;
						callbacks();
						// callback(res.books);
					});

					function callbacks(){
						if(count < 2){
							return;
						}
						let reviews = {};
						_.each(summarys.reviews, function(item){
							reviews[item.review.range] = item.review;
						});
						notes.reviews = reviews;
						callback(notes);
					}
				},
				downloadWeReadBook: function(book){
					let _this = this;
					_this.loadBookNotes(book, function(res){
						let article = res.book;
						let chapters = res.chapters;
						let notes = res.updated.reverse();
						let reviews = res.reviews;

						let tpls = {
							article: '# {title} \n\n',
							banner: '\n ![]({cover} "") \n\n'
						};

						let md = CRS.subs(tpls.article, article);

						if(article.cover){
							md += CRS.subs(tpls.banner, article);
						}

						//章节映射
						let chapterMap = {};
						_.each(chapters, function(item){
							chapterMap[item.chapterUid] = item;
						});

						_.each(notes, function(item){
							let chapterUid = item.bookmarkId.split('_')[1] || '';
							md += '\n\n## ' + chapterMap[chapterUid].title;
							md += '\n' + item.markText;
							if(reviews[item.range]){
								md += '\n > ' + reviews[item.range].content;
							}
						});	

						md += '\n\n\n';
						// md += CRS.subs(tpls.notes, notes);

						// console.log('weread', md);

						let zkCard = CRS.dateFormat(new Date(), 'yyyyMMddhhmm');
						let fileName = article.title || zkCard;
							fileName = fileName + '-书摘.md';
						let file = new File([ md ], fileName, { "type" : "text\/plain" });
					    let objectUrl = URL.createObjectURL(file);
						console.log('weread', objectUrl);
						_this.download(objectUrl, fileName);
					});
				},
				shareWeReadBook: function(book){
					let _this = this;
					if(!_this.token){
						_this.isLogin = true;
						return;
					}
					_this.loadBookNotes(book, function(res){
						let article = res.book;
						let notes = res.updated;
						let reviews = res.reviews;

						let paragraphs = [];
						_.each(notes, function(item, index){
							let summary = '';
							if(reviews[item.range]){
								summary = reviews[item.range].content;
							}
							paragraphs.push({
								note: item.markText,
								summary: summary,
								markTime: item.createTime,
	        					sort: index
							});
						});
						article.paragraphs = paragraphs;

						let host = 'weread.qq.com';
						let uniqueId = md5(host + article.bookId);
						article.uniqueId = uniqueId;

						let originUrl = 'https://weread.qq.com/web/search/books?author=' + encodeURIComponent(article.author);
						article.originUrl = originUrl;
						article.banner = article.cover;

						article.extra = "";

						feynmanRequest({
							api: 'Article.add',
							type: 'request',
							token: _this.token,
							data: article
						}, function(res){
							console.log('Article.add', res);

							feynmanRequest({
								api: 'Article.publish',
								token: _this.token,
								type: 'request',
								data: res.data
							}, function(res){
								// console.log('Article.publish', data, res);
								let params = {
									account: _this.userInfo.account,
									uniqueId: uniqueId
								};
								let link = 'https://notes.bluetech.top/published/{account}/{uniqueId}.html';
								link = CRS.subs(link, params);
								window.open(link);
							});
						});
					});
				},
				getTitle: function(){
					let h1 = document.querySelector('h1');
					let h2 = document.querySelector('h2');
					let h3 = document.querySelector('h3');
					let title = document.title;
						title = title || h1.innerText || h2.innerText  || h3.innerText;
					return title;
				},
				setBanner: function(refresh){
					//从页面中选择一个有效的且高宽 > 200 的图片
					let banner = CRS.cache.get('banner') || '';
					if(banner && !refresh){
						this.article.banner = banner;
						return;
					}

					let imgs = document.querySelectorAll('img');
					let banners = [];
					for(var i=0;i<imgs.length;i++){
					if(imgs[i].src && imgs[i].clientHeight >= 200 && imgs[i].clientWidth >= 200){
							banners.push(imgs[i].src);
						}
					}
					//填充 1 张，后续可以考虑 AI 生成。
					if(banners.length == 0){
						let index = encodeURIComponent(location.href).length%10 + 1;
						let bannerSystem = 'http://pages.bluetech.top/banners/' + index + '.png';
						banners.push(bannerSystem);
					}
					this.banners.data = banners;

					let index = this.banners.index;
					let next = (index + 1)%banners.length;
					this.article.banner = banners[next];
					this.banners.index = next;

					CRS.cache.set('banner', banners[next]);
				},
				setArticleInfo: function(){
					let _this = this;
					let title = _this.getTitle();
					let originUrl = location.href;
					this.setBanner(false);
					this.article.title = title;
					this.article.originUrl = originUrl;
				},
				downloadArticle: function(){
					var documentClone = document.cloneNode(true);
					var article = new Readability(documentClone).parse();
					let html = article.content;
					let md = html2md(html);

					let zkCard = CRS.dateFormat(new Date(), 'yyyyMMddhhmm');
					let fileName = this.article.title || zkCard;
						fileName = fileName + '-原文.md';
					let file = new File([ md ], fileName, { "type" : "text\/plain" });
				    let objectUrl = URL.createObjectURL(file);
					this.download(objectUrl, fileName);
				},
				createMD: function(){
					let tpls = {
						article: '# {title} \n\n 原文：{originUrl} \n',
						banner: '\n ![]({banner} "") \n\n',
						tags: '[[{tag}]]',
						notes: '\n\n {text} \n > {tip}',
					};

					let article = this.article;
					let md = CRS.subs(tpls.article, article);

					let tags = CRS.trim.both(article.tag || '');
						tags = tags.split('，').join(',');
						tags = tags.split(',');
					let tagMap = {};
					if(tags.length > 1){
						md += '\n';
						_.each(tags, function(item){
							if(CRS.trim.both(item) && !tagMap[item]){
								tagMap[item] = item;
								md += '[[' + item + ']] ';							
							}
						});	
					}

					if(article.banner){
						md += CRS.subs(tpls.banner, article);
					}

					let notes = this.notes;
					_.each(notes, function(item){
						md += '\n\n ' + item.text;
						if(item.tip){
							md += '\n > ' + (item.tip || '');
						}
					});	

					md += '\n\n\n';
					// md += CRS.subs(tpls.notes, notes);

					return md;
				},
				downloadNotes: function(){
					let md = this.createMD();

					if(this.notes.length == 0){
						alert('无笔记数据');
						return;
					}

					let zkCard = CRS.dateFormat(new Date(), 'yyyyMMddhhmm');
					let fileName = this.article.title || zkCard;
						fileName = fileName + '-笔记.md';
					let file = new File([ md ], fileName, { "type" : "text\/plain" });
				    let objectUrl = URL.createObjectURL(file);
					this.download(objectUrl, fileName);
				},
				download: function(objectUrl, fileName) {
				    const tmpLink = document.createElement("a");
				    tmpLink.href = objectUrl;
				    tmpLink.download = fileName;
				    document.body.appendChild(tmpLink);
				    tmpLink.click();

				    document.body.removeChild(tmpLink);
				    URL.revokeObjectURL(objectUrl);
				},
				scrollToHL: function(note){
					console.log('scrollToHL' ,note);
					let tipX = note.x;
					let tipY = note.y; //number
					if(tipY <= 0){
						return;
					}
					console.log('scrollToHL-y-x' , tipY, tipX);
					window.scrollTo({
					  top: tipY - 100,
					  left: tipX,
					  behavior: 'smooth'
					});
				},
				noteSaveToServer: function(callback){
					let _this = this;
					let data = this.article;
					let notes = this.notes;

					let paragraphs = [];
					_.each(notes, function(item){
						paragraphs.push({
							note: item.text,
							summary: item.tip || '',
							markTime: item.time,
        					sort: item.y
						});
					});
					data.paragraphs = paragraphs;

					let host = location.host;
					let uniqueId = md5(data.title + host);
					data.uniqueId = uniqueId;

					let pageId = this.pageId;
					let extra = CRS.cache.get(pageId);
					data.extra = JSON.stringify(extra);

					feynmanRequest({
						api: 'Article.add',
						type: 'request',
						token: _this.token,
						data: data
					}, function(res){
						console.log('Article.add', data, res);
						callback(res.data);
					});
				},
				notesPublish: function(data, callback){
					let _this = this;
					feynmanRequest({
						api: 'Article.publish',
						token: _this.token,
						type: 'request',
						data: data
					}, function(res){
						console.log('Article.publish', data, res);
						let params = {
							account: _this.userInfo.account,
							uniqueId: data.uniqueId
						};
						let link = 'https://notes.bluetech.top/published/{account}/{uniqueId}.html';
						link = CRS.subs(link, params);
						callback(link);
					});
				},
				notesRefresh: function(data){
					data = data || {};
					this.notes = data.steps || [];
				},
				initPagenote: function () {
					let _this = this;
					let pageId = _this.pageId;
				    // 如何使用 就看这里就好了，使用前，记得先引入 SDK
				    window.pagenote = new window.PageNote('Feynman',{
				        functionColors:[ // 支持扩展的功能按钮区，
				        ],
				        categories:[],
				        brushes:[ // 画笔
				            {
				                bg:'#FF6900', // rgb 颜色值
				                shortcut:'p', // 快捷键，可选
				                label:'一级画笔', // 说明
				                level:1, // 暂不支持
				            }
				        ],
				        showBarTimeout: 0, // 延迟功能时间 单位毫秒
				        renderAnnotation: function (data, light) { 
				        	// 自定义笔记渲染逻辑，这里可以处理为从服务器端根据 lightId 查询数据来渲染，包括点赞量等数据
				            const element = document.createElement('div');
				            const {tip,lightId,time} = data;
				            const aside = document.createElement('div');
				            aside.innerHTML = `<pagenote-block aria-controls="aside-info">
				            ${new Date(time).toLocaleDateString()}
				            </pagenote-block>`;
				            element.appendChild(aside);

				            element.ondblclick = function () {
				            	console.log(light);
				                light.openEditor();
				            };

				            const asides = [];
				            return [null,asides]
				        },
				        debug: false,
				        enableMarkImg: true,
				    });

				    // 这里可以从服务器端拉取数据，用于 init
				    const data = CRS.cache.get(pageId) || {};
				    console.log('recovery notes:', data);
				    pagenote.init(data);  // 初始化开始工作
				    _this.notesRefresh(data);

				    pagenote.addListener(function (status) {
				        if(status===pagenote.CONSTANT.SYNCED){
				            // 数据变化回调，将数据发送到服务器端，在这里处理
				            // localStorage.setItem('shiuse-notes',JSON.stringify(pagenote.plainData))
				            console.log('note update', pagenote.plainData);
				            CRS.cache.set(pageId, pagenote.plainData);
				            _this.notesRefresh(pagenote.plainData);
				        }
				    })
				},
				createPublicLink: function(){
					let _this = this;
					let md = this.createMD();
					let zkCard = CRS.dateFormat(new Date(), 'yyyyMMddhhmm');
					if(this.token){
						_this.noteSaveToServer(function(res){
							_this.notesPublish(res, function(link){
								window.open(link);
							});
						});
					}else{
						_this.isLogin = true;
					}
				},
				loadTags: function(userInfo){
					let _this = this;
					console.log('Tag.list token', userInfo, _this.token);
					feynmanRequest({
						api: 'Tag.list',
						type: 'request',
						token: _this.token,
						data: {
							tagStatus: 'ENABLE'
						}
					}, function(res){
						console.log('Tag.list', res, res.code);
						console.log('token', _this.token);
						res = res || {};
						if(res.code == '40101'){
							_this.logout();
							return;
						};
						_this.tags = res.data || [];
					});
				},
				userLoginSubmit: function(){
					let _this = this;
					_this.userLogin(function(){
						_this.noteSaveToServer(function(res){
							_this.notesPublish(res, function(link){
								window.open(link);
							});
						});
					});
				},
				userLogin: function(callback){
					let _this = this;
					let userInfo = _this.userInfo;
					console.log(userInfo);
					if(!userInfo.account || !userInfo.password){
						alert('请输入用户名和密码')
						return;
					};
					// userInfo.password = md5(userInfo.password);

					//用户名正则，4到16位（字母，数字，下划线，减号）
					var accountPattern = /^[a-zA-Z0-9_-]{6,50}$/;
					if(!accountPattern.test(userInfo.account)){
						alert('用户名只允许使用字母、数字、下划线和减号，长度为 6~50');
						return;
					}

					var pswPattern = /^[a-zA-Z0-9_-]{10,50}$/;
					if(!pswPattern.test(userInfo.password)){
						alert('密码只允许使用字母、数字、下划线和减号，长度为 10~50');
						return;
					}
					
					feynmanRequest({
						api: 'User.login',
						type: 'request',
						data: userInfo
					}, function(res){
						console.log('User.login', res);
						if(res.code != '0'){
							alert(res.msg);
							return;
						}

						feynmanRequest({
							type: 'setCookie',
							name: 'userInfo',
							value: res.data
						}, function(){
							_this.isLogin = false;
							_this.token = res.data.token;
							_this.account = res.data.account;
							callback();
						});
					});
				},
				logout: function(){
					this.token = '';
					let userInfo = this.userInfo;
					userInfo.token = '';
					
					feynmanRequest({
						type: 'setCookie',
						name: 'userInfo',
						value: userInfo
					}, function(){
					});
				}
			}
		});
	}
})();




